@@ -53,30 +53,30 @@ class HGClientTestSetups(unittest.TestCase):
         os.makedirs(remote_path)
 
         # create a "remote" repo
-        subprocess.check_call(["hg", "init"], cwd=remote_path)
-        subprocess.check_call(["touch", "fixed.txt"], cwd=remote_path)
-        subprocess.check_call(["hg", "add", "fixed.txt"], cwd=remote_path)
-        subprocess.check_call(["hg", "commit", "-m", "initial"], cwd=remote_path)
+        subprocess.check_call("hg init", shell=True, cwd=remote_path)
+        subprocess.check_call("touch fixed.txt", shell=True, cwd=remote_path)
+        subprocess.check_call("hg add fixed.txt", shell=True, cwd=remote_path)
+        subprocess.check_call("hg commit -m initial", shell=True, cwd=remote_path)
         
-        po = subprocess.Popen(["hg", "log", "--template", "'{node|short}'", "-l1"], cwd=remote_path, stdout=subprocess.PIPE)
+        po = subprocess.Popen("hg log --template '{node|short}' -l1", shell=True, cwd=remote_path, stdout=subprocess.PIPE)
         self.local_version_init = po.stdout.read().rstrip("'").lstrip("'")
         # in hg, tagging creates an own changeset, so we need to fetch version before tagging
-        subprocess.check_call(["hg", "tag", "test_tag"], cwd=remote_path)
+        subprocess.check_call("hg tag test_tag", shell=True, cwd=remote_path)
 
         
         # files to be modified in "local" repo
-        subprocess.check_call(["touch", "modified.txt"], cwd=remote_path)
-        subprocess.check_call(["touch", "modified-fs.txt"], cwd=remote_path)
-        subprocess.check_call(["hg", "add", "modified.txt", "modified-fs.txt"], cwd=remote_path)
-        subprocess.check_call(["hg", "commit", "-m", "initial"], cwd=remote_path)
-        po = subprocess.Popen(["hg", "log", "--template", "'{node|short}'", "-l1"], cwd=remote_path, stdout=subprocess.PIPE)
+        subprocess.check_call("touch modified.txt", shell=True, cwd=remote_path)
+        subprocess.check_call("touch modified-fs.txt", shell=True, cwd=remote_path)
+        subprocess.check_call("hg add modified.txt modified-fs.txt", shell=True, cwd=remote_path)
+        subprocess.check_call("hg commit -m initial", shell=True, cwd=remote_path)
+        po = subprocess.Popen("hg log --template '{node|short}' -l1", shell=True, cwd=remote_path, stdout=subprocess.PIPE)
         self.local_version_second = po.stdout.read().rstrip("'").lstrip("'")
         
-        subprocess.check_call(["touch", "deleted.txt"], cwd=remote_path)
-        subprocess.check_call(["touch", "deleted-fs.txt"], cwd=remote_path)
-        subprocess.check_call(["hg", "add", "deleted.txt", "deleted-fs.txt"], cwd=remote_path)
-        subprocess.check_call(["hg", "commit", "-m", "modified"], cwd=remote_path)
-        po = subprocess.Popen(["hg", "log", "--template", "'{node|short}'", "-l1"], cwd=remote_path, stdout=subprocess.PIPE)
+        subprocess.check_call("touch deleted.txt", shell=True, cwd=remote_path)
+        subprocess.check_call("touch deleted-fs.txt", shell=True, cwd=remote_path)
+        subprocess.check_call("hg add deleted.txt deleted-fs.txt", shell=True, cwd=remote_path)
+        subprocess.check_call("hg commit -m modified", shell=True, cwd=remote_path)
+        po = subprocess.Popen("hg log --template '{node|short}' -l1", shell=True, cwd=remote_path, stdout=subprocess.PIPE)
         self.local_version = po.stdout.read().rstrip("'").lstrip("'")
 
         self.local_path = os.path.join(directory, "local")
@@ -163,6 +163,15 @@ class HGClientTest(HGClientTestSetups):
         new_version = self.local_version_second
         self.assertTrue(client.update(new_version))
         self.assertEqual(client.get_version(), new_version)
+
+        self.assertTrue(client.update())
+        self.assertEqual(client.get_version(), self.local_version)
+
+        self.assertTrue(client.update(new_version))
+        self.assertEqual(client.get_version(), new_version)
+
+        self.assertTrue(client.update(''))
+        self.assertEqual(client.get_version(), self.local_version)
        
 
 class HGDiffStatClientTest(HGClientTestSetups):
@@ -174,8 +183,8 @@ class HGDiffStatClientTest(HGClientTestSetups):
         client = HgClient(self.local_path)
         client.checkout(url)
         # after setting up "local" repo, change files and make some changes
-        subprocess.check_call(["rm", "deleted-fs.txt"], cwd=self.local_path)
-        subprocess.check_call(["hg", "rm", "deleted.txt"], cwd=self.local_path)
+        subprocess.check_call("rm deleted-fs.txt", shell=True, cwd=self.local_path)
+        subprocess.check_call("hg rm deleted.txt", shell=True, cwd=self.local_path)
         f = io.open(os.path.join(self.local_path, "modified.txt"), 'a')
         f.write(u'0123456789abcdef')
         f.close()
@@ -188,7 +197,7 @@ class HGDiffStatClientTest(HGClientTestSetups):
         f = io.open(os.path.join(self.local_path, "added.txt"), 'w')
         f.write(u'0123456789abcdef')
         f.close()
-        subprocess.check_call(["hg", "add", "added.txt"], cwd=self.local_path)
+        subprocess.check_call("hg add added.txt", shell=True, cwd=self.local_path)
 
     def tearDown(self):
         pass
